# Theremino System  -  module_theremino.py  -  www.theremino.com
# Tested with Python 2.7.3 in Windows and Linux.
# Tested with Python 3.10 in Windows.

import mmap
import struct
import platform
import math

class _Theremino(object):

   _FLOAT_FORMAT = 'f'
   _FLOAT_SIZE = struct.calcsize(_FLOAT_FORMAT)
   _is_windows = (platform.system() == "Windows")

   # --------------------------------------------------------------------------------------------------------------
   #  NAN numbers - As explained in the following pages:
   # --------------------------------------------------------------------------------------------------------------
   #  https://stackoverflow.com/questions/37093485/how-to-interpret-4-bytes-as-a-32-bit-float-using-python
   #  https://stackoverflow.com/questions/37093485/how-to-interpret-4-bytes-as-a-32-bit-float-using-python/37093610
   # --------------------------------------------------------------------------------------------------------------
   _NAN_Sleep     = struct.unpack(_FLOAT_FORMAT, b'\x01\x00\xC0\x7f')[0]
   _NAN_Reset     = struct.unpack(_FLOAT_FORMAT, b'\x02\x00\xC0\x7f')[0]  # (reset Stepper destination to zero)
   _NAN_Run       = struct.unpack(_FLOAT_FORMAT, b'\x03\x00\xC0\x7f')[0]
   _NAN_Stop      = struct.unpack(_FLOAT_FORMAT, b'\x04\x00\xC0\x7f')[0]
   _NAN_Recognize = struct.unpack(_FLOAT_FORMAT, b'\x05\x00\xC0\x7f')[0]
   _NAN_Reconnect = struct.unpack(_FLOAT_FORMAT, b'\x06\x00\xC0\x7f')[0]
   _NAN_Calibrate = struct.unpack(_FLOAT_FORMAT, b'\x07\x00\xC0\x7f')[0]
   # --------------------------------------------------------------------------------------------------------------

   if not _is_windows:
      #from ctypes import *
      from ctypes import CDLL
      from ctypes import c_int
      from ctypes import c_float
      theremino_slots = CDLL("libThereminoSlots.so")
      _slot_init = theremino_slots.the_slot_init
      _slot_init.argtypes = []
      _slot_init.restype = c_int
      _slot_read = theremino_slots.the_slot_read
      _slot_read.argtypes = [c_int]
      _slot_read.restype = c_float
      _slot_write = theremino_slots.the_slot_write
      _slot_write.argtypes = [c_int, c_float]
      _slot_write.restype = c_int
      _slot_exit = theremino_slots.the_slot_exit
      _slot_exit.argtypes = []
      _slot_exit.restype = c_int

   def __init__(self):
      if self._is_windows:
         self._shmem = mmap.mmap(0, 4080, "Theremino1")
      else:
         self._slot_init()

   def _is_nan(cls, x):
      try:
         return math.isnan(x)
      except:
         return False

   def is_windows(cls):
      return cls._is_windows

   def read_slot(cls, slot):
      if slot < 0 or slot > 999:
         return 0.0
      elif cls._is_windows:
         cls._shmem.seek(slot * cls._FLOAT_SIZE)
         return struct.unpack(cls._FLOAT_FORMAT, \
                              cls._shmem.read(cls._FLOAT_SIZE))[0]
      else:
         return cls._slot_read(slot)

   def read_slot_no_nan(cls, slot):
      value = cls.read_slot(slot)
      if cls._is_nan(value):
         return 0.0
      else:
         return value

   def read_slot_no_nan_0_1000(cls, slot):
      value = cls.read_slot(slot)
      if cls._is_nan(value):
         return 0.0
      elif value < 0.0:
         return 0.0
      elif value > 1000.0:
         return 1000.0
      else:
         return value

   def write_slot(cls, slot, value):
      if slot < 0 or slot > 999:
         return
      elif cls._is_windows:
         cls._shmem.seek(slot * cls._FLOAT_SIZE)
         cls._shmem.write(struct.pack(cls._FLOAT_FORMAT, value))
      else:
         cls._slot_write(slot, value)

   def fill_with_nan_sleep(cls):
      for slot in range(0, 999):
         cls.write_slot(slot, cls._NAN_Sleep)

   def slot_exit(cls):
      if cls._is_windows:
         cls._shmem.close()
      else:
         cls._slot_exit()

Theremino = _Theremino()



